﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Net.NetworkInformation;
using System.Text;

namespace PingM
{
    class Program
    {
        static void Main(string[] args)
        {
            // make sure all args are lowercase
            string[] commandArgs = args;
            for(int i = 0; i < args.Length; i++)
            {
                commandArgs[i] = args[i].ToLower();
            }
            int hostIndex = Array.IndexOf(commandArgs, "-h");
            string[] ipMinMax;
            if (args == null || args.Count() == 0 || args[0] == "" || commandArgs[0] == "/?" || commandArgs[0] == "help")
            {
                Console.WriteLine("");
                Console.WriteLine("PingM provides command-line ping packets that are targeted at multiple Ip Addresses");
                Console.WriteLine("Usage:");
                Console.WriteLine("PingM [/?] [Ip Address] [Ip Address]");
                Console.WriteLine("");
                Console.WriteLine("Options:");
                Console.WriteLine("\t /? or help                 | This page");
                Console.WriteLine("\t [IpAddress] [IpAddress]    | Pings the provided range of Ip\'s");
                Console.WriteLine("\t -h                         | Display returned packets only");
                Console.WriteLine("");
                Console.WriteLine("Note that large ranges of Ip addresses can take a long time");

            }
            else
            {


                Console.WriteLine("A range has been supplied, verifying Ip Addresses...");
                if (Array.IndexOf(args, '-') == -1)
                {
                    ipMinMax = args;
                }
                else
                {
                    ipMinMax = args[0].Split('-');
                }
                IPAddress firstIp;
                IPAddress secondIp;
                IPAddress.TryParse(ipMinMax[0], out firstIp);
                IPAddress.TryParse(ipMinMax[1], out secondIp);
                Console.WriteLine("Input addresses {0} and {1} verified successfully..", firstIp.ToString(), secondIp.ToString());
                string[] firstSegments = firstIp.ToString().Split('.');
                string[] lastSegments = secondIp.ToString().Split('.');

                int[] seg1 = new int[2];
                int[] seg2 = new int[2];
                int[] seg3 = new int[2];
                int[] seg4 = new int[2];

                seg1[0] = Convert.ToInt32(firstSegments[0]);
                seg2[0] = Convert.ToInt32(firstSegments[1]);
                seg3[0] = Convert.ToInt32(firstSegments[2]);
                seg4[0] = Convert.ToInt32(firstSegments[3]);
                seg1[1] = Convert.ToInt32(lastSegments[0]);
                seg2[1] = Convert.ToInt32(lastSegments[1]);
                seg3[1] = Convert.ToInt32(lastSegments[2]);
                seg4[1] = Convert.ToInt32(lastSegments[3]);

                int seg1diff = Math.Abs(seg1[0] - seg1[1]);
                int seg2diff = Math.Abs(seg2[0] - seg2[1]);
                int seg3diff = Math.Abs(seg3[0] - seg3[1]);
                int seg4diff = Math.Abs(seg4[0] - seg4[1]);

                long rawNumber = powerOf(seg1diff, 3) + powerOf(seg2diff, 2) + powerOf(seg3diff, 1) + seg4diff;
                Console.WriteLine("Total number of addressess between {0} and {1}: {2}", firstIp, secondIp, rawNumber);
                Ping controlPing = new Ping();
                PingOptions options = new PingOptions();
                string data = "aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa";
                byte[] buffer = Encoding.ASCII.GetBytes(data);
                int timeout = 120;
                for (int i = 0; i < rawNumber; i++)
                {
                    // Dns Lookup Logic
                    string ipCheck = seg1[0] + "." + seg2[0] + "." + seg3[0] + "." + seg4[0];
                    try
                    {
                        PingReply replyData = controlPing.Send(ipCheck, timeout, buffer, options);
                        if (hostIndex != -1)
                        {
                            if(replyData.Status == IPStatus.Success)
                            {
                                Console.WriteLine("Pinging {0} was {1}", ipCheck, replyData.Status);
                            }
                        }
                        else
                        {
                            Console.WriteLine("Pinging {0} was {1}", ipCheck, replyData.Status);
                        }
                        
                    }
                    catch (ArgumentException e)
                    {
                        Console.WriteLine(e.Message);
                        seg4[0]++;
                        continue;
                    }

                    // Increment the Ip
                    if (seg4[0] < 255)
                    {
                        seg4[0]++;
                    }
                    else
                    {
                        seg4[0] = 0;
                        if (seg3[0] < 255)
                        {
                            seg3[0]++;
                        }
                        else
                        {
                            seg3[0] = 0;
                            if (seg2[0] < 255)
                            {
                                seg2[0]++;
                            }
                            else
                            {
                                seg2[0] = 0;
                                if (seg1[0] < 255)
                                {
                                    seg1[0]++;
                                }
                                else
                                {
                                    throw new Exception("Segment 1 exceeded maximum value");
                                }
                            }
                        }


                    }
                }
            }
            Console.ReadLine();
        }
        private static long powerOf(int inDiff, int totalSuffixBytes)
        {
            long returnTotal;
            double tempTotal;
            double tempPower;
            tempPower = Math.Log(inDiff, 2);
            tempTotal = Math.Pow(2, (Convert.ToDouble(totalSuffixBytes) * 8) + tempPower);
            returnTotal = Convert.ToInt64(tempTotal);
            return returnTotal;
        }
    }

}
